FoldChain

Struct FoldChain 

pub struct FoldChain<T, D: Clone, Settings: FoldSettings<T, D>> { /* private fields */ }
A base FoldChain. See FoldChain.

Implementations

Source

impl<T, D: Clone, Settings: FoldSettings<T, D>> FoldChain<T, (usize, D), SettingsWithSize<Settings>>

pub fn as_fold_list(self) -> FoldList<T, D, Settings>

Convert a FoldChain that keeps track of size to a FoldList.

Source

impl<T, D: Clone, OP: Fn(D, D) -> D + Copy, DeltaOf: Fn(&T) -> D + Copy, Empty: Fn() -> D + Copy> FoldChain<T, D, FoldSettingsStruct<T, D, TupleFun<OP>, DeltaOf, EmptyFn<Empty>>>

pub fn new(op: OP, delta_of: DeltaOf, empty_delta_fn: Empty) -> Self

Create a new empty FoldChain, given the closures for Settings.

pub fn from_iter( op: OP, delta_of: DeltaOf, empty_delta_fn: Empty, iter: impl Iterator<Item = T>, ) -> Self

Create a new FoldChain, given the closures for Settings, and fill it using an iterator, from left to right.

Source

impl<T, D: Clone, OP: Fn(D, D) -> D + Copy, DeltaOf: Fn(&T) -> D + Copy> FoldChain<T, Option<D>, FoldSettingsStruct<T, Option<D>, OptOpFun<TupleFun<OP>>, SomeFun<DeltaOf>, NoneFun>>

pub fn new_with_opt(op: OP, delta_of: DeltaOf) -> Self

Create a new empty FoldChain, given the closures for Settings, except the one for the empty delta, which will always be None.

The resulting delta type will be Option<D> instead of D.

pub fn new_with_opt_from_iter( op: OP, delta_of: DeltaOf, iter: impl Iterator<Item = T>, ) -> Self

Create a new empty FoldChain, given the closures for Settings, except the one for the empty delta, which will always be None, and fill it using an iterator, from left to right.

The resulting delta type will be Option<D> instead of D.

Source

impl<T, D: Clone, Settings: FoldSettings<T, D>> FoldChain<T, D, Settings>

pub fn from_settings(settings: Settings) -> Self

Create a new empty FoldChain, with the specified Settings.

pub fn get_current_simplification(&self)

pub fn get_settings(&self) -> Settings

An alias of get_settings.

pub fn as_imm( &self, ) -> ImmFoldChainSliceStruct<'_, False, True, True, Settings, (), T, D>

An alias of as_imm.

pub fn borrow( &self, ) -> ImmFoldChainSliceStruct<'_, False, True, True, Settings, (), T, D>

An alias of borrow.

pub fn view_drop_left_until( &self, predicate: impl Fn(&D) -> bool, ) -> ImmFoldChainSliceStruct<'_, False, False, True, Settings, (), T, D>

An alias of view_drop_left_until.

pub fn view_take_right_until( &self, predicate: impl Fn(&D) -> bool, ) -> ImmFoldChainSliceStruct<'_, False, False, True, Settings, (), T, D>

An alias of view_take_right_until.

pub fn view_drop_right_until( &self, predicate: impl Fn(&D) -> bool, ) -> ImmFoldChainSliceStruct<'_, False, True, False, Settings, (), T, D>

An alias of view_drop_right_until.

pub fn view_take_left_until( &self, predicate: impl Fn(&D) -> bool, ) -> ImmFoldChainSliceStruct<'_, False, True, False, Settings, (), T, D>

An alias of view_take_left_until.

pub fn view_drop<Predicate: Fn(&D) -> bool, Range: SingleEndedRange<Predicate>>( &self, range: Range, ) -> <<Range as SingleEndedRange<Predicate>>::EndIsLeft as Bool>::IfElse<ImmFoldChainSliceStruct<'_, False, True, False, Settings, (), T, D>, ImmFoldChainSliceStruct<'_, False, False, True, Settings, (), T, D>>

An alias of view_drop.

pub fn view_take<Predicate: Fn(&D) -> bool, Range: SingleEndedRange<Predicate>>( &self, range: Range, ) -> <<Range as SingleEndedRange<Predicate>>::EndIsLeft as Bool>::IfElse<ImmFoldChainSliceStruct<'_, False, False, True, Settings, (), T, D>, ImmFoldChainSliceStruct<'_, False, True, False, Settings, (), T, D>>

An alias of view_take.

pub fn view_reversed( &self, ) -> ImmFoldChainSliceStruct<'_, True, True, True, Settings, (), T, D>

An alias of view_reversed.

pub fn view_with_simplification<NewSimplification: FoldSimplification<T, D>>( &self, new_simplification: NewSimplification, ) -> ImmFoldChainSliceStruct<'_, False, True, True, Settings, <NewSimplification as FoldSimplification<T, D>>::ComposeAfterOther<D, ()>, T, D>

pub fn view_simplify<'a, D2: Clone + 'a, Simplifier: for<'x> Fun<&'x D, D2> + Copy + 'a, OP2: Fun<(D2, D2), D2> + Copy + 'a>( &'a self, simplifier: Simplifier, simplified_op: OP2, ) -> ImmFoldChainSliceStruct<'a, False, True, True, Settings, SimplificationWithoutShortcut<T, D, D2, Simplifier, OP2>, T, D>

An alias of view_simplify.

pub fn view_simplify_with_shortcut<'a, D2: Clone + 'a, Simplifier: for<'x> Fun<&'x D, D2> + Copy + 'a, OP2: Fun<(D2, D2), D2> + Copy + 'a, EmptyShortcut: Fun<(), D2> + Copy + 'a, DeltaShortcut: for<'x> Fun<&'x T, D2> + Copy + 'a>( &'a self, simplifier: Simplifier, simplified_op: OP2, empty_shortcut: EmptyShortcut, delta_shortcut: DeltaShortcut, ) -> ImmFoldChainSliceStruct<'a, False, True, True, Settings, SimplificationWithShortcut<T, D, D2, Simplifier, OP2, EmptyShortcut, DeltaShortcut>, T, D>

pub fn view_unsimplify( &self, ) -> ImmFoldChainSliceStruct<'_, False, True, True, Settings, (), T, D>

An alias of view_unsimplify.

pub fn fold(&self) -> D

An alias of fold.

pub fn is_empty(&self) -> bool

An alias of is_empty.

pub fn left<'b>(&'b self) -> Option<&'b T>

An alias of left.

pub fn right<'b>(&'b self) -> Option<&'b T>

An alias of right.

pub fn foreach(&self, f: impl FnMut(&T))

An alias of foreach.

pub fn iter<'b>(&'b self) -> Iter<'b, False, T, D>

An alias of iter.

pub fn as_mut( &mut self, ) -> MutFoldChainSliceStruct<'_, False, True, True, T, D, Settings, ()>

An alias of as_mut.

pub fn borrow_mut( &mut self, ) -> MutFoldChainSliceStruct<'_, False, True, True, T, D, Settings, ()>

An alias of borrow_mut.

pub fn mut_view_drop_left_until( &mut self, predicate: impl Fn(&D) -> bool, ) -> MutFoldChainSliceStruct<'_, False, False, True, T, D, Settings, ()>

pub fn mut_view_take_right_until( &mut self, predicate: impl Fn(&D) -> bool, ) -> MutFoldChainSliceStruct<'_, False, False, True, T, D, Settings, ()>

pub fn mut_view_drop_right_until( &mut self, predicate: impl Fn(&D) -> bool, ) -> MutFoldChainSliceStruct<'_, False, True, False, T, D, Settings, ()>

pub fn mut_view_take_left_until( &mut self, predicate: impl Fn(&D) -> bool, ) -> MutFoldChainSliceStruct<'_, False, True, False, T, D, Settings, ()>

pub fn mut_view_drop<Predicate: Fn(&D) -> bool, Range: SingleEndedRange<Predicate>>( &mut self, range: Range, ) -> <<Range as SingleEndedRange<Predicate>>::EndIsLeft as Bool>::IfElse<MutFoldChainSliceStruct<'_, False, True, False, T, D, Settings, ()>, MutFoldChainSliceStruct<'_, False, False, True, T, D, Settings, ()>>

An alias of mut_view_drop.

pub fn mut_view_take<Predicate: Fn(&D) -> bool, Range: SingleEndedRange<Predicate>>( &mut self, range: Range, ) -> <<Range as SingleEndedRange<Predicate>>::EndIsLeft as Bool>::IfElse<MutFoldChainSliceStruct<'_, False, False, True, T, D, Settings, ()>, MutFoldChainSliceStruct<'_, False, True, False, T, D, Settings, ()>>

An alias of mut_view_take.

pub fn mut_view_reversed( &mut self, ) -> MutFoldChainSliceStruct<'_, True, True, True, T, D, Settings, ()>

An alias of mut_view_reversed.

pub fn mut_view_with_simplification<NewSimplification: FoldSimplification<T, D>>( &mut self, new_simplification: NewSimplification, ) -> MutFoldChainSliceStruct<'_, False, True, True, T, D, Settings, <NewSimplification as FoldSimplification<T, D>>::ComposeAfterOther<D, ()>>

pub fn mut_view_simplify<'a, D2: Clone + 'a, Simplifier: for<'x> Fun<&'x D, D2> + Copy + 'a, OP2: Fun<(D2, D2), D2> + Copy + 'a>( &'a mut self, simplifier: Simplifier, simplified_op: OP2, ) -> MutFoldChainSliceStruct<'a, False, True, True, T, D, Settings, SimplificationWithoutShortcut<T, D, D2, Simplifier, OP2>>

An alias of mut_view_simplify.

pub fn mut_view_simplify_with_shortcut<'a, D2: Clone + 'a, Simplifier: for<'x> Fun<&'x D, D2> + Copy + 'a, OP2: Fun<(D2, D2), D2> + Copy + 'a, EmptyShortcut: Fun<(), D2> + Copy + 'a, DeltaShortcut: for<'x> Fun<&'x T, D2> + Copy + 'a>( &'a mut self, simplifier: Simplifier, simplified_op: OP2, empty_shortcut: EmptyShortcut, delta_shortcut: DeltaShortcut, ) -> MutFoldChainSliceStruct<'a, False, True, True, T, D, Settings, SimplificationWithShortcut<T, D, D2, Simplifier, OP2, EmptyShortcut, DeltaShortcut>>

pub fn mut_view_unsimplify( &mut self, ) -> MutFoldChainSliceStruct<'_, False, True, True, T, D, Settings, ()>

An alias of mut_view_unsimplify.

pub fn pop_left(&mut self) -> Option<T>

An alias of pop_left.

pub fn pop_right(&mut self) -> Option<T>

An alias of pop_right.

pub fn append_left(&mut self, value: T)

An alias of append_left.

pub fn append_right(&mut self, value: T)

An alias of append_right.

pub fn set_left_or_err(&mut self, value: T) -> Result<T, T>

An alias of set_left_or_err.

pub fn set_right_or_err(&mut self, value: T) -> Result<T, T>

An alias of set_right_or_err.

pub fn set_left(&mut self, value: T) -> T

An alias of set_left.

pub fn set_right(&mut self, value: T) -> T

An alias of set_right.

pub fn update_left<R>(&mut self, f: impl FnOnce(Option<&mut T>) -> R) -> R

An alias of update_left.

pub fn update_right<R>(&mut self, f: impl FnOnce(Option<&mut T>) -> R) -> R

An alias of update_right.

pub fn foreach_mut(&mut self, f: impl FnMut(&mut T))

An alias of foreach_mut.

pub fn take_all(&mut self) -> FoldChain<T, D, Settings>

An alias of take_all.

pub fn append_all_right(&mut self, chain: FoldChain<T, D, Settings>)

An alias of append_all_right.

pub fn append_all_left(&mut self, chain: FoldChain<T, D, Settings>)

An alias of append_all_left.

pub fn append_left_from_iter(&mut self, iter: impl Iterator<Item = T>)

An alias of append_left_from_iter.

pub fn append_right_from_iter(&mut self, iter: impl Iterator<Item = T>)